package net.sf.sageplugins.sageimdb;


import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Container class to get person information.
 * use get() method
 * @author Niel
 *
 */
public class ImdbWebGetPerson {

	/** 
	 * background updater class to get filmographies
	 * @author Niel
	 *
	 */
	static private class UpdateEpisodeFilmog extends ImdbWebBgUpdateThread{
		
		
	    UpdateEpisodeFilmog(DbPersonObject obj,ImdbWebObjectRef ref) {
	        super(obj,ref,"EpsiodeFilmog");
	        start();
	    }
	    /* (non-Javadoc)
	     * @see net.sf.sageplugins.sageimdb.ImdbWebBgUpdateThread#doUpdate()
	     */
	    void doUpdate() throws Exception {
	        DbPersonObject obj=(DbPersonObject)this.obj;
	        URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"filmoseries");
	        ImdbWebPageReader reader=new ImdbWebPageReader(url);
	        if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
	        	throw new DbFailureException("Failed to get Person Episode Filmography Page");
	        
	        final Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
			Matcher contentMat=contentPat.matcher(reader.getPageText());
	        if ( !contentMat.find()){
	        	throw new DbFailureException("Failed to find content in Person Episode Filmography Page");
	        }

	        /* 
			 * Pattern that finds a series block, and puts:
			 *  Series name into $1
			 */
			final Pattern filmogPat=Pattern.compile(
					"<div class=\"filmo\">" +
					"\\s*<span>" +
					// $1 - series name
					"\\s*<a href=\"/title/[^\"]*\">&#x22;([^<]*)&#x22;</a>" + 
					".*?</span>" +
					"\\s*<ol>" +
					// $2 -- epsiodes
					"(.*?)" +
					"</ol>\\s*?" +
					"</div>");
			final Pattern episodePat=Pattern.compile(
					"<li>.*?" +
					// $1  - ref, $2 - Episode name $3 - Character Name
					"<a href=\"([^\"]+)\"[^>]*>(.*?</a>[^-]*)-(.*?)" +
					"</li>");
	        
	        Matcher filmogMat=filmogPat.matcher(contentMat.group());
	        Filmography filmog=null;
	        
	        // copy filmographies, update copy, replace original...
	        // avoids concurrent modification exceptions
	        Vector newFilmographies=new Vector(obj.filmographies);
        	// loop tho series
        	while ( filmogMat.find()) {
        		if ( filmog==null){
        			filmog=new Filmography("TV Episodes");
        			newFilmographies.add(filmog);
        		}
        		String seriesName=ImdbWebPageReader.deHtmlIze(filmogMat.group(1));

        		Matcher episodeMat=episodePat.matcher(filmogMat.group(2));
        		// loop thro episodes
        		while ( episodeMat.find()){
        			String episodeName=ImdbWebPageReader.deHtmlIze(episodeMat.group(2));
        			Role role=new Role(new ImdbWebObjectRef(
        					DbObjectRef.DB_TYPE_TITLE,
        					seriesName + "\n-- " + episodeName,
        					episodeMat.group(1)),
        					ImdbWebPageReader.deHtmlIze(episodeMat.group(3)));
        			filmog.addRole(role);
        		}
        	}
        	// replace with updated
        	obj.filmographies=newFilmographies;
        }
	}

	/**
	 * Background updater class to get Bio page
	 * @author Niel
	 *
	 */
	static private class UpdateBio extends ImdbWebBgUpdateThread{
		
	    
		UpdateBio(DbPersonObject obj,ImdbWebObjectRef ref) {
	        super(obj,ref,"Bio");
	        start();
	    }
	    /* (non-Javadoc)
	     * @see net.sf.sageplugins.sageimdb.ImdbWebBgUpdateThread#doUpdate()
	     */
	    void doUpdate() throws Exception {
	        DbPersonObject obj=(DbPersonObject)this.obj;
	        URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"bio");
	        ImdbWebPageReader reader=new ImdbWebPageReader(url);

	        if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
	        	throw new DbFailureException("Failed to get Person Episode Filmography Page");
	        
	        final Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
			Matcher contentMat=contentPat.matcher(reader.getPageText());
	        if ( !contentMat.find()){
	        	throw new DbFailureException("Failed to find content in Person Episode Filmography Page");
	        }
	        
	        // Bio sections start with <h5> and end with <br/>
			final Pattern sectionPat=Pattern.compile(
					"<h5[^>]*>" +
					// $1 section title 
					"(.*?)" +
					"</h5>" +
					// $2 section content
					"(.*?)" +
					"<br/>");
			Matcher sectionMat=sectionPat.matcher(contentMat.group());
	        while ( sectionMat.find()){
	        	String title=ImdbWebPageReader.deHtmlIze(sectionMat.group(1));
	        	if ( title.equalsIgnoreCase("Birth name")) {
	        		obj.birthname=ImdbWebPageReader.deHtmlIze(sectionMat.group(2));
	        	} else if ( title.equalsIgnoreCase("Nickname")) {
	        		obj.nickname=ImdbWebPageReader.deHtmlIze(sectionMat.group(2));
	        	} else if ( title.equalsIgnoreCase("Height")) {
	        		obj.height=ImdbWebPageReader.deHtmlIze(sectionMat.group(2));
	        	} else if ( title.equalsIgnoreCase("Mini biography")) {
	        		if ( obj.biography == null ) {
	        			obj.biography=ImdbWebPageReader.deHtmlIze(sectionMat.group(2).replaceAll("<br/?>"," \r\n"));
	        		} else {
	        			obj.biography=obj.biography+"\r\n----\r\n"+ImdbWebPageReader.deHtmlIze(sectionMat.group(2).replaceAll("<br/?>"," \r\n"));
	        		}
	        	} else if ( title.equalsIgnoreCase("Trade mark")) {
	        		obj.trademark=ImdbWebPageReader.deHtmlIze(sectionMat.group(2).replaceAll("</p><p","\r\n \r\n<p "));
	        	} else if ( title.equalsIgnoreCase("Trivia")) {
	        		obj.trivia=ImdbWebPageReader.deHtmlIze(sectionMat.group(2).replaceAll("</p><p","\r\n \r\n<p "));
	        	} else if ( title.equalsIgnoreCase("Personal quotes")) {
	        		obj.quotes=ImdbWebPageReader.deHtmlIze(sectionMat.group(2).replaceAll("</p><p","\r\n \r\n<p "));
	        	} else if ( title.equalsIgnoreCase("Spouse")) {
	        		// spouses listed in table...
	        		obj.married=ImdbWebPageReader.deHtmlIze(sectionMat.group(2).replaceAll("</tr>","\r\n"));
	        	}
	        }
	    }
	}
	
	public static DbPersonObject get(ImdbWebObjectRef ref) throws DbFailureException {
		DbPersonObject obj=new DbPersonObject(ref);
        boolean getEpisodeFilmography=false;

        try {
			URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"filmotype");
			obj.imdbUrl=url.toString();
			
			ImdbWebPageReader reader=new ImdbWebPageReader(url);
	        if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
	        	throw new DbFailureException("Failed to get Person Episode Filmography Page");
	        

			// got text, need to get...
			// Name
			// Picture
			// DOB
			// Filmographies
			

			// then launch background updaters for
			// Tv Episode Filmographies
			// Bio

	        
	    	final Pattern namePat=Pattern.compile(
	    			"<div +id=\"tn15title\"> *?"+
	    			"<h1>([^<]*)</h1>");
	    	
	    	final Pattern episodeListLinkPat=Pattern.compile(
	    			"<div +id=\"tn15lhs\">.*<a href=\"filmoseries\" class=\"link\">");
	    	
	    	final Pattern picturePat=Pattern.compile(
	    			"<div +id=\"tn15lhs\"> *?"+
	    			"<div +class=\"photo\"> *?" +
	    			"<a name=\"headshot\"[^>]*> *?" +
	    			"<img [^>]* src=\"([^\"]*)\"");
	    	
	    	final Pattern dobPat=Pattern.compile(
	    			"<div class=\"info\"> *?<h5>Date of Birth:</h5> *?" +
	    			"(.*?)" +
	    			"(<a class=\"tn15more inline\" href=\"bio\">more</a>)? *?</div>");
	    	

	    	
	    	/** 
	    	 * pattern to match a group of filmographies
	    	 */
	    	final Pattern filmogGroupPat=Pattern.compile(
	    			"<div[^>]*class=\"filmo\"[^>]*?> *" +
	    			"<h5> *" +
	    			// $1 - filmog type
	    			"<a name=\"([^\"]*)\"> *" +
	    			// $2 - Filmog type printable
	    			"([^>]*): *" +
	    			"</a> *" +
	    			"</h5>.*?" + 
	    			"<ol>" +
	    			// $3 -- elements
	    			"(.*?)" +
	    			"</ol>.*?" +
	    			"</div>");


	    	/**
	    	 * Pattern to match a filmography item
	    	 */
	    	final Pattern filmogItemPat=Pattern.compile(
	    			"<li>.*?" +
	    			// $1 IMDB ref
	    			"<a[^>]*href=\"(/title/[^\"]*)\"[^>]*> *" +
	    			// $2 details -- title, role etc 
	    			"(.*?)" +
	    			"</li>");


	     
	        Matcher matcher=namePat.matcher(reader.getPageText());
	        if ( matcher.find())
	        	obj.name=ImdbWebPageReader.deHtmlIze(matcher.group(1));
	        
	        matcher=picturePat.matcher(reader.getPageText());
	        if ( matcher.find())
	        	try {
	        		obj.imageURL=new URL(matcher.group(1));
	        	} catch (MalformedURLException e) {
	        		System.out.println("IMDB invalid image URL for "+obj.name+" - "+matcher.group(1));
	        	}
		    matcher=episodeListLinkPat.matcher(reader.getPageText());
		    getEpisodeFilmography=matcher.find();
		        
		    Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
			matcher=contentPat.matcher(reader.getPageText());
	        if ( !matcher.find()){
	        	throw new DbFailureException("Failed to find content in Person Episode Filmography Page");
	        }
	        String content=matcher.group();

	        // DOB
	        matcher=dobPat.matcher(content);
	        if ( matcher.find()){
	        	obj.dob=ImdbWebPageReader.deHtmlIze(matcher.group(1));
	        }

	    	/**
	    	 * Pattern to get details from a filmography pattern with has a '....' before the role
	    	 */
	    	final Pattern filmogDetailsPatRole=Pattern.compile(
	    			// $1 everything before '....'
	    			"^(.*?)" +
	    			// $2 tv series/mini series
	    			"(<small>.*?</small>)?"+
	    			// $3 imdbpro info
	    			"(<i>\\(<a href=\"http://pro.imdb.com/[^\"]*\">.*?</a>\\)</i>)?"+
	    			// ....
	    			"\\s*\\.{4}\\s*" +
	    			// $4 extraneous links
	    			"(<a onclick[^>]*\">)?" +
	    			// $5 role
	    			"([^(]*)" +
	    			// $6 episode info
	    			"(\\(\\d+ episodes?, [^)]*\\))?" +
	    			// $7 extra info -- episode, aka etc
	    			"(<br/>.*)?");
	    	
	    	/**
	    	 * Pattern to get the details for the first episode in and episode list
	    	 */
	    	final Pattern filmogFirstEpsiodeInfo=Pattern.compile(
	    			// $1 title $2 year and $3 (TV/VG/V/etc)
	    			"([^<]*</a>(\\s*\\([0-9?]{4}[^)]*\\)\\s*)?(\\(\\p{javaUpperCase}+\\))?)" +
	    			"<small>TV epsiode</small>"
	    			);

	    	/**
	    	 * Pattern to get the details for a filmography that does not have a '....'
	    	 * eg... 
	    	 * 
	    	 */
	    	final Pattern filmogDetailsPatNoRole=Pattern.compile(
	    			// $1 title,  $2 optional (YYYY/IV) or (????),  $3 optional (TV/VG/V/etc)
	    			"^([^<]*)</a>(\\s*\\([0-9?]{4}[^)]*\\))?\\s*(\\(\\p{javaUpperCase}+\\))?" +
	    			// $4 optional <small>tv series/mini series<small>
	    			"(\\s*<small>.*?</small>)?"+
	    			// $5 optional imdbpro info
	    			"(\\s*<i>\\(<a href=\"http://pro.imdb.com/[^\"]*\">.*?</a>\\)</i>)?"+
	    			// $6 optional (credit)
	    			"(\\s*\\([^)]*\\))?" +
	    			// $7 optional episode info
	    			"(\\s*\\(\\d+ episodes?, [^)]*\\))?" +
	    			// $8 optional extra info -- episode, aka etc
	    			"(\\s*<br/>.*)?");


	    	
	        // Filmographies.
	        Matcher filmogGroupMatcher=filmogGroupPat.matcher(content);
	        while (filmogGroupMatcher.find()){
	        	// got a filmography group
	        	Filmography filmog=new Filmography(ImdbWebPageReader.deHtmlIze(filmogGroupMatcher.group(2)));
	        	obj.filmographies.add(filmog);

	        	// now look at each filmography item 
	        	Matcher filmogItemMatcher=filmogItemPat.matcher(filmogGroupMatcher.group(3));
	        	while ( filmogItemMatcher.find()) {
	        		// got a filmography item..
	        		// $1 is ref
	        		String imdbRef=filmogItemMatcher.group(1);
	        		// $2 is info -- to be parsed
	        		// if '...' has role:
	        		
	        		if (filmogItemMatcher.group(2).indexOf("....")>=0){ 
	        			Matcher detailsRoleMatcher=filmogDetailsPatRole.matcher(filmogItemMatcher.group(2));
	        			detailsRoleMatcher.find();
	        			// $1 title $2 year and $3 (TV/VG/V/etc)

	        			String title=ImdbWebPageReader.deHtmlIze(detailsRoleMatcher.group(1));

	        			String rolePlayed;
	        			if (detailsRoleMatcher.group(5)!= null)
	        				{rolePlayed=ImdbWebPageReader.deHtmlIze(detailsRoleMatcher.group(5));
	        				rolePlayed=rolePlayed.replaceAll("/ ...", "");
	        				}
	        			else 
	        				rolePlayed="";

	        			// check for episodes...
	        			if ( detailsRoleMatcher.group(6)!=null 
	        					&& detailsRoleMatcher.group(7) != null 
	        					&& detailsRoleMatcher.group(6).length()>0) {
	        				if ( detailsRoleMatcher.group(6).startsWith("(1 episode,")) {
	        					// need to get episode title from $7
	        					Matcher firstEp=filmogFirstEpsiodeInfo.matcher(detailsRoleMatcher.group(7));
	        					firstEp.find();
	        					title=title+" - "+ImdbWebPageReader.deHtmlIze(firstEp.group(1));
	        					Role role=new Role(new ImdbWebObjectRef(
	        							DbObjectRef.DB_TYPE_TITLE,
	        							title,
	        							imdbRef),
	        							rolePlayed);
	        					filmog.addRole(role);
	        				} else {
	        					// multiple episodes -- summarize series info only
	        					Role role=new Role(new ImdbWebObjectRef(
	        							DbObjectRef.DB_TYPE_TITLE,
	        							title + " " +detailsRoleMatcher.group(6),
	        							imdbRef),
	        							rolePlayed);
	        					filmog.addRole(role);
	        				}
	        			} else {
	        				// no episodes
	        				Role role=new Role(new ImdbWebObjectRef(
	        						DbObjectRef.DB_TYPE_TITLE,
	        						title ,
	        						imdbRef),
	        						rolePlayed);
	        				filmog.addRole(role);
	        			}
	        		} else {
	        			
	        			Matcher detailsNoRoleMatcher=filmogDetailsPatNoRole.matcher(filmogItemMatcher.group(2));
	        			if ( detailsNoRoleMatcher.find()) {
	        				// $1 title $2 year and $3 (TV/VG/V/etc)

	        				String title=ImdbWebPageReader.deHtmlIze(
	        						detailsNoRoleMatcher.group(1));
	        				if (detailsNoRoleMatcher.group(2) !=null )
	        					title=title+" "+ImdbWebPageReader.deHtmlIze(
		        						detailsNoRoleMatcher.group(2));
	        				if (detailsNoRoleMatcher.group(3) !=null )
	        					title=title+" "+ImdbWebPageReader.deHtmlIze(
		        						detailsNoRoleMatcher.group(3));
	        				String credit;
	        				if ( detailsNoRoleMatcher.group(6) != null )
	        					credit=ImdbWebPageReader.deHtmlIze(
	        						detailsNoRoleMatcher.group(6));
	        				else
	        					credit="";
	        				
	        				// check for epsiodes...
	        				if ( detailsNoRoleMatcher.group(7)!=null 
	        						&& detailsNoRoleMatcher.group(8) != null 
	        						&& detailsNoRoleMatcher.group(7).length()>0) {
	        					if ( detailsNoRoleMatcher.group(7).startsWith("(1 episode,")) {
	        						// need to get episode title from $8
	        						Matcher firstEp=filmogFirstEpsiodeInfo.matcher(detailsNoRoleMatcher.group(8));
	        						firstEp.find();
	        						title=title+" - "+ImdbWebPageReader.deHtmlIze(firstEp.group(1));
	        						Role role=new Role(new ImdbWebObjectRef(
	        								DbObjectRef.DB_TYPE_TITLE,
	        								title,
	        								imdbRef),
	        								credit);
	        						filmog.addRole(role);
	        					} else {
	        						// multiple episodes -- summarize series info only
	        						Role role=new Role(new ImdbWebObjectRef(
	        								DbObjectRef.DB_TYPE_TITLE,
	        								title + " " +detailsNoRoleMatcher.group(5),
	        								imdbRef),
	        								credit);
	        						filmog.addRole(role);
	        					}
	        				} else {
	        					// no episodes
	        					Role role=new Role(new ImdbWebObjectRef(
	        							DbObjectRef.DB_TYPE_TITLE,
	        							title ,
	        							imdbRef),
	        							credit);
	        					filmog.addRole(role);
	        				}                			
	        			} else {
	        				System.out.println("Failed to match filmography details for "+obj.name+" - "+filmogItemMatcher.group(2));
	        			}
	        		}

	        	}
	        }


        } catch (Throwable e){
        	System.out.println(e);
        	e.printStackTrace();
        	throw new DbFailureException("Failed to get details for person ID "+ref.getImdbRef(),e);        	
        }

        // launch BG updaters
        if ( getEpisodeFilmography)
        	new UpdateEpisodeFilmog(obj,ref);
        new UpdateBio(obj,ref);
        return obj;
	}

}
