package net.sf.sageplugins.sageimdb;

import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.sageplugins.sageutils.Translate;



public class ImdbWebGetTitle {

	
	/** 
	 * background updater class to get episodes
	 * @author Niel
	 *
	 */
	static class UpdateEpisodes extends ImdbWebBgUpdateThread{
		
	    UpdateEpisodes(DbTitleObject obj,ImdbWebObjectRef ref) {
	        super(obj,ref,"Epsiodes");
	        start();
	    }
	    /* (non-Javadoc)
	     * @see net.sf.sageplugins.sageimdb.ImdbWebBgUpdateThread#doUpdate()
	     */
	    void doUpdate() throws Exception {
	        DbTitleObject obj=(DbTitleObject)this.obj;
	        URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"episodes");
	        
	        ImdbWebPageReader reader=new ImdbWebPageReader(url);
	        if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
	        	throw new DbFailureException("Failed to get Title Episode List Page "+url+" - "+reader.getResponseCode()+" - "+reader.getResponseMessage());

	        Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
	        Matcher contentMat=contentPat.matcher(reader.getPageText());
	        if ( !contentMat.find()){
	        	throw new DbFailureException("Failed to find content in Title Episode List Page");
	        }
	        
	        Pattern episodePat=Pattern.compile(
					// $1 = series/ep info, $2 = imdbref, $3=episode name
					"<h3>([^<]*)<a href=\"(/title/[^\"]*)\"[^>]*>([^<]*)</a>\\s*</h3>" +
					// $2 = OAD
					"<span[^>]*>(Original Air Date.*?)<br/?>"	
					);
	        
	        Vector episodes=new Vector();
	        Matcher episodeMat=episodePat.matcher(contentMat.group());
	        while (episodeMat.find()){
	        	String episodeName=ImdbWebPageReader.deHtmlIze(episodeMat.group(3));
	        	
                Role role=new Role(new ImdbWebObjectRef(
                        DbObjectRef.DB_TYPE_TITLE,
                        "Episode: " + episodeName,
                        episodeMat.group(2)),
                        ImdbWebPageReader.deHtmlIze(episodeMat.group(1)) + " - " + 
                        ImdbWebPageReader.deHtmlIze(episodeMat.group(4))
                		);
                episodes.add(role);
	        }
	        if ( episodes.size()>0)
	        	obj.episodes=episodes;
	    }
	}

	    
	/** 
	 * background updater class to get locations
	 * @author Niel
	 *
	 */
	static class UpdateLocations extends ImdbWebBgUpdateThread{


		UpdateLocations(DbTitleObject obj,ImdbWebObjectRef ref) {
			super(obj,ref,"Locations");
			start();
		}
		/* (non-Javadoc)
		 * @see net.sf.sageplugins.sageimdb.ImdbWebBgUpdateThread#doUpdate()
		 */
		void doUpdate() throws Exception {
			DbTitleObject obj=(DbTitleObject)this.obj;
			URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"locations");

			ImdbWebPageReader reader=new ImdbWebPageReader(url);
			if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
				throw new DbFailureException("Failed to get Title Locations Page "+url+" - "+reader.getResponseCode()+" - "+reader.getResponseMessage());

	        Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
			Matcher contentMat=contentPat.matcher(reader.getPageText());
			if ( !contentMat.find()){
				throw new DbFailureException("Failed to find content in Title Locations Page");
			}
			
			Pattern locationsPat=Pattern.compile(
					// $1 -- location
					"<dt>\\s*<a href=\"/List[^\"]*\"[^>]*>(.*?)</a>\\s*</dt>" +
					// $2 location extra inf
					"\\s*<dd>(.*?)</dd>"	
			);
			StringBuffer locations=new StringBuffer();
			Matcher locationsMat=locationsPat.matcher(contentMat.group());
			while (locationsMat.find()){
				locations.append(ImdbWebPageReader.deHtmlIze(locationsMat.group(1)));

				String extrainf=ImdbWebPageReader.deHtmlIze(locationsMat.group(2));
				if ( extrainf.length()>0) {
					locations.append(" - ");
					locations.append(extrainf);
				}
				locations.append("\n ---- \n");
			}
			if ( locations.length()>0)
				obj.locations=locations.toString();
		}
	}

	static class UpdateQuotes extends ImdbWebBgUpdateThread{
		
		
		UpdateQuotes(DbTitleObject obj,ImdbWebObjectRef ref) {
			super(obj,ref,"Quotes");
			start();
		}
		/* (non-Javadoc)
		 * @see net.sf.sageplugins.sageimdb.ImdbWebBgUpdateThread#doUpdate()
		 */
		void doUpdate() throws Exception {
			DbTitleObject obj=(DbTitleObject)this.obj;
			URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"quotes");
			ImdbWebPageReader reader=new ImdbWebPageReader(url);
			if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
				throw new DbFailureException("Failed to get Title quotes Page "+url+" - "+reader.getResponseCode()+" - "+reader.getResponseMessage());

	        Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
			Matcher contentMat=contentPat.matcher(reader.getPageText());
			if ( !contentMat.find()){
				throw new DbFailureException("Failed to find content in Title Locations Page");
			}
			
			Pattern quotesPat=Pattern.compile(
					"<a name=\"qt[^\"]*\"[^>]*>(.*?)<hr\\s"
					);
			StringBuffer quotes=new StringBuffer();
			Matcher quotesMat=quotesPat.matcher(contentMat.group());
			while (quotesMat.find()){
				String quote=ImdbWebPageReader.deHtmlIze(
						quotesMat.group(1).replaceAll("<br/?>", "\n"));
				quotes.append(quote.trim());
				quotes.append("\n ---- \n");
			}
			if ( quotes.length()>0)
				obj.quotes=quotes.toString();
		}
	}

	static class UpdateSummary extends ImdbWebBgUpdateThread{
		
		UpdateSummary(DbTitleObject obj,ImdbWebObjectRef ref) {
			super(obj,ref,"Summary");
			start();
		}
		/* (non-Javadoc)
		 * @see net.sf.sageplugins.sageimdb.ImdbWebBgUpdateThread#doUpdate()
		 */
		void doUpdate() throws Exception {
			DbTitleObject obj=(DbTitleObject)this.obj;
			URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"plotsummary");
			ImdbWebPageReader reader=new ImdbWebPageReader(url);
			if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
				throw new DbFailureException("Failed to get Title summaries Page "+url+" - "+reader.getResponseCode()+" - "+reader.getResponseMessage());

	        Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
			Matcher contentMat=contentPat.matcher(reader.getPageText());
			if ( !contentMat.find()){
				throw new DbFailureException("Failed to find content in Title summaries Page");
			}

			// <div class="info">
			// <h5>Plot Synopsis:</h5>
			// John McClane, a man struggling with his marriage, gets trapped in a high-rise building by a gang of &#34;terrorists&#34; planning a major robbery...
			// <a class="tn15more inline" href="synopsis">more (warning! contains spoilers)</a>
			//
			// </div>
			Pattern synopsisPat=Pattern.compile(
					"<div\\s*class=\"info\">\\s*" +
					"<h5>Plot Synopsis:</h5>(.*?)" +
					"(<a class=\"tb15more inline.*?</a>)?" +
					"\\s*</div>");

			/*
			 * <p class="plotpar">
			 * In the city of Los Angeles, a Christmas party is held on the 30th floor of the Nakatomi Plaza Hotel. While the party is going on, downstairs, a band of German terrorists arrive and take the entire building hostage including its employees and attempts a huge robbery. But the only one who eludes capture is New York City Cop John McClane who launches a one man war in an attempt to stop the terrorists and save all hostages including his wife Holly.
			 * <i>
	 		 * Written by
	 		 * <a href="/SearchPlotWriters?Shaun%20Ouimette%20%7Bxtreemshaun589@hotmail.com%7D">Shaun Ouimette {xtreemshaun589@hotmail.com}</a>
			 * </i>
			 * </p>
			 */
			Pattern summaryPat=Pattern.compile(
					"<p class=\"plotpar\">" +
					"(.*?)" +
					"(<i>Written by.*?</i>)?" +
					"</p>"
					);

			
			StringBuffer summaries=new StringBuffer();

			// first get small synopsis, then full summary
			Matcher synopMat=synopsisPat.matcher(contentMat.group());
			if ( synopMat.find()){
				String synop=ImdbWebPageReader.deHtmlIze(synopMat.group(1));
				if ( ! synop.startsWith("This plot synopsis is empty") ) {
					summaries.append(synop);
					summaries.append("\n ----\n");
				}
			}

			// now add each summary...
			Matcher summaryMat=summaryPat.matcher(contentMat.group());
			while (summaryMat.find()){
				summaries.append(ImdbWebPageReader.deHtmlIze(
						summaryMat.group(1).replaceAll("<br/?>", "\n")));
				summaries.append('\n');
				summaries.append(ImdbWebPageReader.deHtmlIze(
									summaryMat.group(2)));
				summaries.append("\n ---- \n");
			}
			if ( summaries.length()>0)
				obj.summaries=obj.summaries+summaries.toString();
		}
	}

	static class UpdateTrivia extends ImdbWebBgUpdateThread {
		
		UpdateTrivia(DbTitleObject obj,ImdbWebObjectRef ref) {
			super(obj,ref,"Trivia");
			start();
		}
		/* (non-Javadoc)
		 * @see net.sf.sageplugins.sageimdb.ImdbWebBgUpdateThread#doUpdate()
		 */
		void doUpdate() throws Exception {
			DbTitleObject obj=(DbTitleObject)this.obj;
			URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"trivia");
			ImdbWebPageReader reader=new ImdbWebPageReader(url);
			if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
				throw new DbFailureException("Failed to get Title trivia Page "+url+" - "+reader.getResponseCode()+" - "+reader.getResponseMessage());

			Pattern triviacontentPat=Pattern.compile("<div\\s*class=\"sodatext\">.*?<br>");
			Matcher triviacontentMat=triviacontentPat.matcher(reader.getPageText());
			if ( !triviacontentMat.find()){
				throw new DbFailureException("Failed to find content in Title Locations Page");
			}

			StringBuffer trivia=new StringBuffer();
			Pattern triviaItemPat=Pattern.compile(
					"class=\"sodatext\">(.*?)(<br/?>)"
					);
			Matcher triviaItemMat=triviaItemPat.matcher(triviacontentMat.group());
			while (triviaItemMat.find()){
				trivia.append(ImdbWebPageReader.deHtmlIze(
						triviaItemMat.group(1).replaceAll("<br/?>", "\n")));
				trivia.append("\n ---- \n");
			}
			if ( trivia.length()>0)
				obj.trivia=trivia.toString();
		}
	}

	public static DbTitleObject get(ImdbWebObjectRef ref) throws DbFailureException{
		DbTitleObject obj=new DbTitleObject(ref);
		try {
			URL url=new URL("http",ImdbWebBackend.IMDB_HOST,-1,ref.getImdbRef()+"combined");

			ImdbWebPageReader reader=new ImdbWebPageReader(url);
			if ( reader.getResponseCode()!=HttpURLConnection.HTTP_OK)
				throw new DbFailureException("Failed to get Title Page "+url+" - "+reader.getResponseCode()+" - "+reader.getResponseMessage());

	        Pattern contentPat=Pattern.compile(ImdbWebBackend.CONTENT_PAT_STRING,Pattern.CASE_INSENSITIVE);
			Matcher contentMat=contentPat.matcher(reader.getPageText());
			if ( !contentMat.find()){
				throw new DbFailureException("Failed to find content in Title Page");
			}
			
			String content=contentMat.group();
			
			// get LHS for photo etc
			boolean getSummary=false;
			boolean getTrivia=false;
			boolean getQuotes=false;
			boolean getEpisodes=false;
			boolean getLocations=false;
			
			Pattern lhsPat=Pattern.compile("<div\\s*id=\"tn15lhs\"(.*?)<div\\s*id=\"tn15main");
			Matcher lhsMat=lhsPat.matcher(reader.getPageText());
			if ( lhsMat.find()) {
				String lhs=lhsMat.group(1);
				// check for sub-pages
				getSummary=lhs.indexOf("href=\"plotsummary\" class=\"link\"")>0;
				getTrivia=lhs.indexOf("href=\"trivia\" class=\"link\"")>0;
				getQuotes=lhs.indexOf("href=\"quotes\" class=\"link\"")>0;
				getEpisodes=lhs.indexOf("href=\"episodes\" class=\"link\"")>0;
				getLocations=lhs.indexOf("href=\"locations\" class=\"link\"")>0;
				
				// get photo URL
				Pattern photoPat=Pattern.compile(
						"<div\\s+class=\"photo\">\\s*?" +
						"<a name=\"poster\"[^>]*>\\s*?" +
						"<img [^>]* src=\"([^\"]*)\"",Pattern.CASE_INSENSITIVE);
				Matcher photoMat=photoPat.matcher(lhs);
				if ( photoMat.find())
					try {
						obj.imageURL=new URL(photoMat.group(1));
					} catch ( MalformedURLException e){
						System.out.println("Failed to get URL of photo from title page: "+photoMat.group(1));
					}
			} else {
				System.out.println("Failed to get LHS from title page");
			}
			obj.imdbUrl=url.toString();

			obj.summaries="";

			// get name, year from title tag
			Pattern titlePat=Pattern.compile("<title>(.*?)(\\([0-9?]{4}[^)]*\\))?</title>");
			Matcher titleMat=titlePat.matcher(reader.getPageText());
			if ( titleMat.find()){
				// remove quotes from title
				obj.name=ImdbWebPageReader.deHtmlIze(titleMat.group(1)); // .replaceAll("\"", "");
				if ( titleMat.group(2)!=null && ! titleMat.group(2).equals("????") )
					// remove brackets from year.
					obj.year=titleMat.group(2).replaceAll("[^0-9]*", "");
			}
			
			// get user rating
			Pattern userRatingPat=Pattern.compile(
					// $2 rating
					"User Rating:.*?([0-9.]+/10)");
			Matcher userRatingMat=userRatingPat.matcher(content);
			if ( userRatingMat.find()){
				obj.rating=userRatingMat.group(1);
			}
			
			Pattern infoBlockPat=Pattern.compile(
					"<div\\s*class=\"info\">" +
					// $1 info title
					"\\s*<h5>(.*?)</h5>" +
					// $2 info content
					"(.*?)" +
					"</div>"
					);
			Matcher infoBlockMat=infoBlockPat.matcher(content);
			while ( infoBlockMat.find()){
				String infoTitle=ImdbWebPageReader.deHtmlIze(infoBlockMat.group(1));
				String infoContent=infoBlockMat.group(2).trim();
				int moreIndex=infoContent.indexOf("<a class=\"tn15more");
				if ( moreIndex>0)
					infoContent=infoContent.substring(0, moreIndex-1).trim();
				if ( infoContent.length()>0){
						
					if ( infoTitle.indexOf("Tagline")>=0) {
						obj.tagLines=ImdbWebPageReader.deHtmlIze(infoContent);
		
					} else if (infoTitle.indexOf("Certification")>=0) {
						obj.usa_rating=Translate.decode(infoContent.replaceAll(".*USA:([^<]+)<.*","$1").trim());					
					
					}else if (infoTitle.indexOf("Plot:")>=0) {
						obj.summaries=ImdbWebPageReader.deHtmlIze(infoContent)+"\n ----\n";
					
					}else if (infoTitle.indexOf("TV Series:")>=0) {
						// <a href="/title/tt0088571/">&#34;Moonlighting&#34; (1985)</a>
						Pattern seriesPat=Pattern.compile(
								"<a\\s*href=\"(/title[^\"]*)\"[^>]*>([^<]*)</a>"
								);
						Matcher seriesMat=seriesPat.matcher(infoContent);
						if (seriesMat.find())
							obj.seriesRef=new ImdbWebObjectRef(
									DbObject.DB_TYPE_TITLE,
									ImdbWebPageReader.deHtmlIze(seriesMat.group(2)),
									seriesMat.group(1)); 
					
					}else if (infoTitle.indexOf("Original Air Date:")>=0 
							|| infoTitle.indexOf("Release Date:")>=0) {
						obj.airingDate=ImdbWebPageReader.deHtmlIze(infoContent);
					
					}else if (infoTitle.indexOf("Runtime:")>=0) {
						obj.runningtime=ImdbWebPageReader.deHtmlIze(infoContent);
					
					}else if (infoTitle.indexOf("Genre:")>=0) {
						obj.genres=ImdbWebPageReader.deHtmlIze(infoContent).split(" \\| ");

					} else if (infoTitle.indexOf("MPAA:") >= 0 && infoContent.indexOf("Rated ") >= 0) {
						   // 	String rated = "Rated ";
							int pos1 = infoContent.indexOf("Rated ");
							int pos2 = infoContent.indexOf(" ", pos1);
							obj.mpaa_rating = ImdbWebPageReader.deHtmlIze(infoContent.substring(pos2, infoContent.length()));
					
					}else if ( infoContent.startsWith("<a href=\"/name/")){
						// top credits
						infoContent=ImdbWebPageReader.deHtmlIze(infoContent.replaceAll("<br/?>", ", "));
						if ( obj.topCredits==null)
							obj.topCredits="";
						obj.topCredits=obj.topCredits+infoTitle+" - "+infoContent+"\n";
					}
				}
			}
									
			// get cast... 
			
			// Split from cast starting point to end of text, 
			// then split from cast ending point, which is first table def after cast table
			int cast_start=content.indexOf("<table class=\"cast\"");
			int cast_end=content.indexOf("<table",cast_start+1);
			if ( cast_start>0 && cast_end > 0){

				Pattern castMemberPat=Pattern.compile(
						"<tr\\s*class=\"[^\"]*\"[^>]*>" +
						"\\s*<td\\s*class=\"hs\"[^>]*>.*?</td>"+
						// $1 = imdb ref
						// $2 = cast name
						"\\s*<td\\s*class=\"nm\"[^>]*>\\s*<a href=\"([^\"]*)\"[^>]*>(.*?)</a></td>"+
						"\\s*<td\\s*class=\"ddd\"[^>]*>.*?</td>"+
						// $3 = part name
						"\\s*<td\\s*class=\"char\"[^>]*>(.*?)</td>"+
						"\\s*</tr>"
						);
				Matcher castMemberMat=castMemberPat.matcher(content.substring(cast_start, cast_end));
				Vector personList=new Vector();
				while ( castMemberMat.find()){
					personList.add(
							new Role(
									new ImdbWebObjectRef(DbObjectRef.DB_TYPE_PERSON,
											ImdbWebPageReader.deHtmlIze(castMemberMat.group(2)), // name
											castMemberMat.group(1)), // imdb ref
											ImdbWebPageReader.deHtmlIze(castMemberMat.group(3)))); // role
				}
				obj.cast=personList;

				// get originators
				int originators_end=content.indexOf("<hr",cast_end);
				if ( originators_end >0 ) {
	
					// person group is a table where first row is title, remaining rows are people
					Pattern origGroupPat=Pattern.compile(
							"<table[^>]*>" +
							"<tr[^>]*>" +
							"<td[^>]*>" +
							// $1 group name
							"<h5[^>]*>(.*?)</h5>.*?" +
							"</td></tr>" +
							//$2 remaining rows
							"(.*?)" +
							"</table>"
							);
					
					// match people with IMDB ref, name and role
					// <tr> <td valign="top">
					// <a href="/name/nm0001532/">John McTiernan</a>
					// </td><td valign="top">&nbsp;</td>
					// <td valign="top"> </td>
					// </tr>
					// 
					// <tr><td valign="top">
					// <a href="/name/nm0861636/">Roderick Thorp</a>
					// </td><td>&nbsp;</td>
					// <td valign="top">(novel &#34;Nothing Lasts Forever&#34;)<br><br></td>
					// </tr>
					
					Pattern origPersonPat=Pattern.compile(
							"<tr[^>]*>" +
							// $1 imdb ref, $2 name
							"\\s*<td[^>]*><a href=\"(/name[^\"]*)\"[^>]*>(.*?)</a>.*?</td>" +
							"\\s*<td[^>]*>.*?</td>" +
							// $3 role, $4 (bracketed info)
							"\\s*<td[^>]*>([^(]*?)(\\([^)]*\\))?</td>"
							);
					
					personList=new Vector();
					
					Matcher origGroupMat=origGroupPat.matcher(content.substring(cast_end, originators_end));
					// for each group
					while ( origGroupMat.find() ){
						String groupName=ImdbWebPageReader.deHtmlIze(origGroupMat.group(1));
						Matcher origPersonMat=origPersonPat.matcher(origGroupMat.group(2));
						
						// for each person
						while ( origPersonMat.find()){
							String name=ImdbWebPageReader.deHtmlIze(origPersonMat.group(2));
							String role=ImdbWebPageReader.deHtmlIze(origPersonMat.group(3));
							String info=ImdbWebPageReader.deHtmlIze(origPersonMat.group(4));
							if ( role.length()>0 )
								role=role+" "+info;
							else
								role="("+groupName+") "+info;
							personList.add(
									new Role(
											new ImdbWebObjectRef(DbObjectRef.DB_TYPE_PERSON,
													name,
													origPersonMat.group(1)), // imdb ref
													role)); // role
						}
					}
					if ( personList.size()>0)
						obj.originators=personList;
				} else {
					System.out.println("Failed to find originators block in "+url);
				}
			} else {
				System.out.println("Failed to find cast block in "+url);
			}
			if ( getSummary )
				new UpdateSummary(obj,ref);
			else if (obj.summaries==null)
				obj.summaries="";
			if ( getTrivia )
				new UpdateTrivia(obj,ref);
			else
				obj.trivia=null;
			if ( getQuotes )
				new UpdateQuotes(obj,ref);
			else
				obj.quotes=null;
			if ( getLocations )
				new UpdateLocations(obj,ref);
			else
				obj.locations=null;
			if ( getEpisodes )
				new UpdateEpisodes(obj,ref);
			else
				obj.episodes=null;

		} catch (DbFailureException e){
			throw e;
		} catch (Throwable e) {
			throw new DbFailureException(e);
		}
		return obj;
	}

}
