package net.sf.sageplugins.sageimdb;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


import net.sf.sageplugins.sageimdb.ImdbWebBackend;
import net.sf.sageplugins.sageimdb.ImdbWebObjectRef;
import net.sf.sageplugins.sageutils.Translate;

public class ImdbWebSearch {

	static ImdbWebSearchCache searchCache = new ImdbWebSearchCache();
	
	// precompiled regexps...
	static Pattern contentPat=Pattern.compile("<div *id=\"pagecontent\".*?<div *?id=\"footer\"",Pattern.CASE_INSENSITIVE);
	static Pattern patResult=Pattern.compile("<td.*?><a href=\"(/title|/name)[^>]*?>[^<].*?</td>",Pattern.CASE_INSENSITIVE);
	static Pattern patTitle = Pattern.compile("<a href=\"(/title/tt[^/]+?/)[^\"]*?\"[^>]*?>([^<]+?)</a>([^<]*)<",Pattern.CASE_INSENSITIVE);
	static Pattern patPerson = Pattern.compile("<a href=\"(/name/nm[^/]+?/)[^\"]*?\"[^>]*?>([^<]+?</a>[ ()IVX]+?)(<small>.*?</small>)",Pattern.CASE_INSENSITIVE);
	static Pattern moreResPat=Pattern.compile("<a +href=\"/(find?[^\"]+?)\".*?>See all the matches</a>",Pattern.CASE_INSENSITIVE);
	
	static Vector search(String searchstring) throws DbFailureException{
		Vector results;
	    results=searchCache.getSearch(searchstring);
	    if ( results==null) {
	    	results=new Vector();
			search(searchstring,results);
	    	if ( results != null && results.size()>0 )
	    		searchCache.addSearch(searchstring,results);
	    }
		return results;
	}

	static private void search(String searchstring,Vector results) throws DbFailureException {
		URL url;
		try {
			url=new URL("http://"+ImdbWebBackend.IMDB_HOST+"/"+searchstring);
		} catch (MalformedURLException e){
			throw new DbFailureException(e.toString(),e);
		}
		ImdbWebPageReader webReader;
		try {
			webReader=new ImdbWebPageReader(url);
		} catch (IOException e) {
			throw new DbFailureException("Failed to get URL "+ url + " - " +e);
		}

		switch (webReader.getResponseCode()) {
		case HttpURLConnection.HTTP_OK:
			// OK
			break;

		case HttpURLConnection.HTTP_MOVED_PERM:
		case HttpURLConnection.HTTP_MOVED_TEMP:
			// redirect to movie/character name... 
			String location=webReader.getHeaderField("Location");
			String searchText="";
			try {
				searchText=URLDecoder.decode(searchstring.replaceAll(".*q=",""),"utf-8");
			} catch (UnsupportedEncodingException e) {
				System.out.println("IMDB Web:"+ e.toString());
			}
				
			if ( location != null ) {
				if ( location.startsWith("http://"+ImdbWebBackend.IMDB_HOST+"/name/nm")) {
					results.add(
							new Role(
									new ImdbWebObjectRef(
											DbObjectRef.DB_TYPE_PERSON,
											searchText,
											location.replaceAll("http://"+ImdbWebBackend.IMDB_HOST+"(/name/nm[^/]+/).*","$1")),
							""));
					return;
				} 
				else if ( location.startsWith("http://"+ImdbWebBackend.IMDB_HOST+"/title/tt")) {
					results.add(
							new Role(
									new ImdbWebObjectRef(
											DbObjectRef.DB_TYPE_TITLE,
											searchText,
											location.replaceAll("http://"+ImdbWebBackend.IMDB_HOST+"(/title/tt[^/]+/).*","$1")),
							""));
					return;
				}
			}
			// if we got here, no valid results from redirect
			throw new DbFailureException("Search failed, redirected to location="+location);

		default:
			throw new DbFailureException("Search failed with status "+webReader.getResponseCode()+" "+webReader.getResponseMessage());
		}


		// if we got here, then HTTP_OK was returned, and we should have some text 


		// Main content is between
		// <table id="outerbody"
		// and
		// <div id="footer"
		Matcher contentMatch=contentPat.matcher(webReader.getPageText());
		if ( ! contentMatch.find() )
			throw new DbFailureException("Failed to find search results content in "+url);

		String content=contentMatch.group();


		// matches are: 
		// <a href="/title/tt0371881/">sdadadsa (2003)</a>

		Matcher matResult=patResult.matcher(content);
		while( matResult.find() && results.size()<ImdbWebBackend.MAX_SEARCH_RESULTS){
			String res=matResult.group();
			String type=matResult.group(1);
			if ( type.equals("/title")) {
				Matcher matchtitle=patTitle.matcher(res);
				if ( matchtitle.find()){
					String name=Translate.decode(
							matchtitle.group(2).trim()+
							matchtitle.group(3));
					results.add(new Role ( new ImdbWebObjectRef(
							DbObjectRef.DB_TYPE_TITLE,
							Translate.decode(name),
							matchtitle.group(1)),
					""));
				} else {
					System.out.println("IMDB failed search title match: "+res);
				}
			} else if ( type.equals("/name")) {
				Matcher matchperson=patPerson.matcher(res);
				if ( matchperson.find()) {
					String name=Translate.decode(
							matchperson.group(2)
							.replaceAll("<[^>]+>","")
							.trim());
					String part=Translate.decode(
							matchperson.group(3)
							.replaceAll("<[^>]+>","")
							.trim());
					results.add(new Role ( new ImdbWebObjectRef(DbObjectRef.DB_TYPE_PERSON,
							Translate.decode(name),
							matchperson.group(1)),
							part));
				} else {
					System.out.println("IMDB failed search person match: "+res);
				}
			}
		} // loop thro matches

		if ( results.size()<ImdbWebBackend.MAX_SEARCH_RESULTS ) {
			// if not many results, and 'more matches' present in search results, search more
			Matcher moreResMat=moreResPat.matcher(content);
			if ( moreResMat.find()){
				search(moreResMat.group(1),results);
			}
		}
	}
}
